//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2011 Florian Eßer <f.esser@rwth-aachen.de>
//

#ifndef ELEVATIONDATAGENERATOR_H
#define ELEVATIONDATAGENERATOR_H

#include <QtCore/QObject>
#include <QProcess>
#include <QThread>

#include "GeoDataLineString.h"
#include "MarbleWidget.h"
#include "HttpDownloadManager.h"
#include "FileStoragePolicy.h"


namespace Marble
{

/**
* @short The class that does the calculation stuff in the background, so the UI keeps responding.
*
*/

class ElevationDataGenerator : public QObject
{
    Q_OBJECT
 public:
    QList<QPointF> m_eleData;
    GeoDataLineString m_points;
    qreal m_maxElevation;
    QStringList m_missingHgtFiles;

    ElevationDataGenerator();

    ~ElevationDataGenerator();

    void setMarbleWidget( MarbleWidget *widget );

    void calculateDistances();

    void calculateElevations();

 Q_SIGNALS:
    void dataUpdated();

 private Q_SLOTS:
    void addToUnzipQueue( QString filename, QString downloadId );
    void unzipFinished( int exitCode );

 private:
    FileStoragePolicy* m_storagePolicy;
    HttpDownloadManager* m_downloadManager;
    QProcess* m_unzipProcess;
    QString m_currentZipFileName;
    QStringList m_unzipQueue;
    MarbleWidget* m_marbleWidget;

    QString latLonToHgtFilename( const GeoDataCoordinates &coordinates );
    QList<qint16> readHgtFile( const QString &filename, const QString &srtm );
    qreal lookupElevation( const GeoDataCoordinates &coordinates, const QList<qint16> &hgtData );
    bool canExecute( const QString &executable ) const;
    void unzipHgt();
	QString	latlongToSrtmArea( const GeoDataCoordinates &coordinates );

};

}

#endif // ELEVATIONDATAGENERATOR_H